#!/bin/bash
#
# NOTE: DO NOT MODIFY THIS FILE (Please see below for more information)
#
# This file contains the initial information necessary to specify the
# contents of the filesystem template for a particular service; e.g.
# siteinfo, apache, ssh, etc. We understand that there will be users
# out there who would like to modify the contents of these filesystem
# templates; handling this in a way that is compatible with RPM's
# behavior during upgrade requires a second file that contains the
# modifications. 
#
# First, understand that all this initial file does is spit out
# information about the proposed contents for the filesystem template,
# with each line containing one piece of information. Thus, in order
# to modify this information, you can create a new file called
# <service>.custom.sh (e.g. siteinfo.custom.sh); the lines of
# information written by this first script will be piped through
# the <service>.custom.sh script, and the resulting output will be
# what is finally considered for the filesystem template. Here is
# a straightforward example of how to add and remove additional data
# (we will remove the 'which' rpm and add the 'xmlto' rpm):
#
# #!/bin/bash
#
# # grep out the which RPM from the input, passing the rest through
# /bin/cat | grep -v '^S,rpm:which'
# # add an additional line specifying the xmlto RPM
# /bin/echo "S,rpm:xmlto"

#
# Begin actual script...
#

#!/bin/bash
cat <<EOF
S,rpm:ImageMagick
N:S,rpm:ImageMagick-c++
N:S,rpm:ImageMagick-perl
S,rpm:MySQL-python
S,rpm:XFree86-Mesa-libGL
S,rpm:XFree86-libs
S,rpm:XFree86-libs-data
S,rpm:apr
S,rpm:apr-util
S,rpm:aspell
S,rpm:aspell-en
S,rpm:bash !/etc/skel/.bash(_logout|_profile|rc)
S,rpm:beecrypt
N:S,rpm:beecrypt-python
S,rpm:bind-utils
S,rpm:bzip2
S,rpm:bzip2-libs
S,rpm:compat-db
N:S,rpm:compat-libgcj
S,rpm:compat-libstdc++
S,rpm:compat-slang
#sh-utils textutils and fileutils now replaced by coreutils
S,rpm:coreutils
S,rpm:cpio
S,rpm:cracklib
S,rpm:cracklib-dicts
S,rpm:crontabs
S,rpm:curl
S,rpm:cyrus-sasl
S,rpm:cyrus-sasl-md5
S,rpm:cyrus-sasl-plain
S,rpm:db4
N:S,rpm:db4-utils
S,rpm:dev /dev$|/dev/pts$
S,rpm:diffutils
S,rpm:dos2unix
S,rpm:e2fsprogs
S,rpm:elfutils-libelf
N:S,rpm:elfutils
S,rpm:expat
S,rpm:file
S,rpm:filesystem
S,rpm:findutils
S,rpm:fontconfig
S,rpm:freetype
S,rpm:ftp
S,rpm:gawk
S,rpm:gd
S,rpm:gdbm
S,rpm:gettext
S,rpm:glib
S,rpm:glib2
S,rpm:glibc !/etc/ld.so.conf|/etc/ld.so.preload|/etc/nsswitch.conf
S,rpm:glibc-common
S,rpm:gmp
S,rpm:gnupg
S,rpm:gpm
S,rpm:grep
S,rpm:groff
S,rpm:gzip
S,rpm:hesiod
S,rpm:info
S,rpm:initscripts !/etc/rc|/etc/sysconfig
S,rpm:iputils
S,rpm:jwhois
S,rpm:krb5-libs
S,rpm:less
S,rpm:libacl
S,rpm:libattr
S,rpm:libgcc
S,rpm:libgcj
N:S,rpm:libgcrypt
S,rpm:libjpeg
S,rpm:libpng
N:S,rpm:libpng10
S,rpm:libstdc++
S,rpm:libtermcap
S,rpm:libtiff
S,rpm:libtool-libs
S,rpm:libungif
S,rpm:libuser
N:S,rpm:libxml
N:S,rpm:libxml2
N:S,rpm:libxslt
S,rpm:logrotate !/etc/logrotate.conf|/etc/cron.daily/logrotate
S,rpm:mailcap
S,rpm:mailx
S,rpm:man
S,rpm:mktemp
S,rpm:mx
N:S,rpm:mysql-devel
N:S,rpm:mysql !/etc/my.cnf
# Provide patterns for MySQL RPMs too
N:S,rpm:MySQL-devel
N:S,rpm:MySQL-shared-compat
N:S,rpm:MySQL-shared
N:S,rpm:MySQL-client
N:S,rpm:MySQL-python
S,rpm:ncftp
S,rpm:ncompress
S,rpm:net-tools !.*bin/(hostname|dnsdomainname)
S,rpm:ncurses
S,rpm:openldap
S,rpm:openssh
S,rpm:openssh-clients
S,rpm:openssl
N:S,rpm:openssl096
N:S,rpm:openssl096b
S,rpm:pam !/etc/pam.d/system-auth
S,rpm:passwd
S,rpm:pcre
S,rpm:perl
S,rpm:perl-Archive-Tar
S,rpm:perl-Compress-Zlib
S,rpm:perl-Convert-TNEF
S,rpm:perl-Crypt-SSLeay
S,rpm:perl-Date-Calc
S,rpm:perl-DateManip
S,rpm:perl-DBD-MySQL
S,rpm:perl-DBD-Pg
S,rpm:perl-DBI
S,rpm:perl-Digest-HMAC
S,rpm:perl-Digest-SHA1
S,rpm:perl-File-MMagic
S,rpm:perl-Filter
S,rpm:perl-HTML-Parser
S,rpm:perl-HTML-Tagset
S,rpm:perl-IO-stringy
S,rpm:perl-libwww-perl
S,rpm:perl-libxml-enno
S,rpm:perl-libxml-perl
S,rpm:perl-MailTools
S,rpm:perl-MIME-tools
S,rpm:perl-Net-DNS
S,rpm:perl-suidperl
S,rpm:perl-TimeDate
S,rpm:perl-Time-HiRes
S,rpm:perl-URI
S,rpm:perl-XML-Dumper
S,rpm:perl-XML-Encoding
S,rpm:perl-XML-Grove
S,rpm:perl-XML-Parser
S,rpm:perl-XML-Twig
S,rpm:pine
S,rpm:popt
S,rpm:postgresql-libs
S,rpm:procmail
S,rpm:python
S,rpm:readline
S,rpm:sed
S,rpm:sendmail !.*(pam.d/smtp|etc/mail|etc/aliases|etc/sysconfig|var/spool)
S,rpm:setup    /etc/(services|protocols)
S,rpm:shadow-utils !/etc/default/useradd|/etc/login.defs
S,rpm:tar
S,rpm:tcp_wrappers !.*tcpd$
S,rpm:termcap
S,rpm:traceroute
S,rpm:tzdata
S,rpm:net-snmp
S,rpm:unixODBC
S,rpm:unzip
S,rpm:util-linux !.*pam.d/login
S,rpm:vim-common
N:S,rpm:vim-enhanced
S,rpm:vim-minimal
S,rpm:webppliance-virtualhosting .*/(pam_ensimvwh.so|virtualname.so|virthostmail)
S,rpm:which
S,rpm:zip
S,rpm:zlib
EOF

if /bin/rpm -q ensim-appliance 1>/dev/null 2>&1; then
  /bin/echo "S,rpm:ensim-appliance /usr/lib/opcenter/cmdline_common"
else
  /bin/echo "S,rpm:ensim-appliance-l /usr/lib/opcenter/cmdline_common"
fi
