#!/usr/bin/eperl
#
# Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
# Exit codes (on failure error message goes to stderr):
#  0 - success
#  1 - failure
# 
# Boolean arguments on the command line are given as 0 or 1. All the
# command line arguments are encoded to avoid problems with escapes.
#
# All functions defined here either return an error message if an
# error occured and "" if everything went well or allways return a
# valid value, but exit (with code 1) printing an error message if an
# error occurs. This second type of functions have _e appended to
# their name. Functions are allowed to print results onto stdout, but
# errors are printed only in the main program.
#

package VHCommon;

require VHAppConf;
require VHConst;
require VHConfiguration;
use IO::Seekable;

use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/virtualhosting";
use lib ($ENV{OCW_SVCPATH} or "/usr/lib/opcenter") . "/cmdline_common";

use ERRORS;
use Carper;

#
# Processes a command line argument that is supposed to be a y/n flag
#
sub Flag_e() {
  my ($flag)=@_;
  if ($flag eq "1") {
    return 1;
  } elsif ($flag eq "0") {
    return 0;
  }
  $cerr += $E_INVALIDFLAG and $cerr += { flag => $flag } ;
    print STDERR "Wrong argument $flag passed where a flag was expected\n";
    exit 1;
}

#
# This function deletes the filesystem used by a user with virtual
# hosting. Returns an error message that is empty on success.
#
sub DeleteFileSystem() {
  my ($user)=@_;
  my ($error_message)="";

  system("/bin/umount $VHConst::VIRTUALFS/$user/dev/pts >/dev/null 2>/dev/null");
  system ("/bin/rm -rf $VHConst::VIRTUALFS/$user >/dev/null 2>/dev/null");
  my($exit_value)  = $? >> 8;
  if ($exit_value ne 0) {
    $cerr += $E_UNABLEDELETEVFS and $cerr += { vfs => "$VHConst::VIRTUALFS/$user" } ;
    $error_message="Error deleting virtual file system $VHConst::VIRTUALFS/$user .\n";
  }
  unlink("/etc/logrotate.d/0virtualhosting.$user") or 
    $cerr += $E_UNABLEDELETEFILE and $cerr += { file => "/etc/logrotate.d/0virtualhosting.$user" } and
    $error_message .= "Cannot delete file /etc/logrotate.d/0virtualhosting.$user .\n";
  unlink("/etc/cron.daily/0virtualhosting.$user") or 
    $cerr += $E_UNABLEDELETEFILE and $cerr += { file => "/etc/cron.daily/0virtualhosting.$user" } and
    $error_message .= "Cannot delete file /etc/cron.daily/0virtualhosting.$user .\n";
  unlink("/etc/cron.weekly/0virtualhosting.$user") or 
    $cerr += $E_UNABLEDELETEFILE and $cerr += { file => "/etc/cron.weekly/0virtualhosting.$user" } and
    $error_message .= "Cannot delete file /etc/cron.weekly/0virtualhosting.$user .\n";
  unlink("/etc/cron.monthly/0virtualhosting.$user") or 
    $cerr += $E_UNABLEDELETEFILE and $cerr += { file => "/etc/cron.monthly/0virtualhosting.$user" } and
    $error_message .= "Cannot delete file /etc/cron.monthly/0virtualhosting.$user .\n";
  #remove all the symlinks from the user's home pointing to the
  #virtual file system
  my(@pwent) = getpwnam($user);
  unless (@pwent) {return $error_message} #Maybe the user is already gone
  my($homedir) = $pwent[7];
  if (opendir (HOMEDIR,$homedir)) { 
    my(@filenames) = map {"$homedir/$_"} readdir(HOMEDIR);
    my(@symlinks) = grep {-l $_} @filenames;
    my($link);
    foreach $link (@symlinks) { #all the symlinks
      my ($value)= readlink ($link);
      if ($value =~ /$VHConst::VIRTUALFS\/$user\//) {
	unlink($link) or 
        $cerr += $E_UNABLEDELETEFILE and $cerr += { file => $link } and 
        $error_message .= "Cannot delete file $link .\n" ;
      }
    }
  } else {
    unless ($error_message) {
      $cerr += $E_UNABLEOPENUSERHOME and 
      $cerr += { home => $homedir , user => $user } and 
      $error_message.="Cannot open home directory $homedir for user $user .\n";
    }
  }
  return $error_message;
}

#
# This function adds an user to the server. The user will be
# associated a new group with the same name.
#
sub AddDomainOwner($$) {

  my($record,$passwd) = @_;

  my $user = $record->{User};
  my $domain = $record->{Domain};
  my $telnet = $record->{Telnet};
  my $ftp = $record->{FTP};

  my(@pwent) = getpwnam($user);
  if (@pwent) {
    $cerr += $E_USEREXISTS and $cerr += { user => $user } ;
    return "The user $user already exists.\n";
  }
  my(@grent) = getgrnam($user);
  if (@grent) {
    $cerr += $E_GRPEXISTS and $cerr += { grp => $user } ;
    return "A group called $user already exists.\n";
  }
  my($shell) = DetermineRootLevelShell($ftp,$telnet);
  my($error_message)="";

  unless (open (ADDUSER, "/usr/sbin/real.useradd -m -s $shell -c \"$domain\" $user 2>&1 |"))
  #Solaris: We'll see
    { $cerr += $E_USERADDERRORS ;
    return "Errors during useradd.\n" ;}
  while (<ADDUSER>) {
    $error_message.=$_;
  }
  unless (close (ADDUSER))
    { $cerr += $E_USERADDERRORS ;
    $error_message.="Errors during useradd.\n" ;}
  # Hack! ignore the "unknown group" error message. ???
  if ($error_message eq "useradd: unknown group \n") {
    $error_message="";
  }
  unless ($error_message) {
    unless(open (CHPASSWD, "|/usr/sbin/chpasswd"))
      { $cerr += $E_UNABLESETPASSWORD ;
      return "Cannot set password.\n";}
    print CHPASSWD "$user:$passwd\n";
    unless (close (CHPASSWD))
      { $cerr += $E_CHPASSWDERR ;
      $error_message.="Errors during chpasswd.\n";}
  }

  my(@rpwent) = getpwnam($user);  #Get the user info of the
                                            #virtual domain owner
  unless(@rpwent) {
    $cerr += $E_NODOMAINOWNER and $cerr += { domain => $domain, user => $user};
    return "Owner of domain $domain , $user does not exist.\n";}
  my ($uid) = ($rpwent[2]);
  my ($gid) = ($rpwent[3]);

  #add the mail box
  AddNBMailBox("$user", $uid, $gid, $record);

  return $error_message;
}

# deletes a domain owner user (and its group) from the server
sub DelDomainOwner() {
  my ($user)=@_;

  my($error_message)="";
  if (open (DELUSER, "/usr/sbin/userdel -r $user 2>&1 |"))
    { 
      while (<DELUSER>) {$error_message.=$_;}
      close (DELUSER) or 
        $cerr += $E_USERDELERRORS and 
        $error_message.="Errors during userdel.\n";
    } 
  else {
    $cerr += $E_USERDELERRORS ;
    $error_message.= "Errors during userdel.\n";}

  # remove the group as well (pr7588)
  if (open (DELGRP, "/usr/sbin/groupdel $user 2>&1 |")) { 
      close (DELGRP);
  } else { 
    $cerr += $E_UNABLESTARTGRPDEL ;
    $error_message.="Couldn't start groupdel process.\n"; }

  my ($inbox)="/var/spool/mail/$user";
  if (lstat($inbox)) {
    unlink ($inbox) or 
        $cerr += $E_UNABLEDELETEINBOX and $cerr += { inbox => $inbox } and
        $error_message.= "Cannot delete inbox $inbox .\n";
  }
  return $error_message;
}

# add a user to a chroot environment
sub AddVirtUser() {
  my ($user,$fullname,$ftpallowed,$telnetallowed,$record)=@_;
  my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";

  my(@pwent) = getpwnam($record->{User});  #Get the user info of the
                                           #virtual domain owner
  unless(@pwent) {
    $cerr += $E_NODOMAINOWNER and 
    $cerr += { domain => $Domain, user => $record->{User} } ;
    return "Owner of domain $Domain , $record->{User} does not exist.\n";}
  my ($gid) = ($pwent[3]);
  my ($shell) = &VHCommon::DetermineShell($ftpallowed,$telnetallowed);
  my ($error_message) = &ActualAddVirtUser($user,$fullname,$shell,$gid,$vroot);
  if ($error_message =~ /^useradd: can\'t get unique uid/) {
    #it might make sense to switch to another virtual uid range
    my ($err);
    if ($err = &VHConfiguration::ReadUIDConfFile()) {
      return $error_message.$err;
    }
    if ($#{$record->{Ranges}}>=0) {
      #OK, we have uid ranges to choose from
      if ($err = &VHConfiguration::ParseDomainPasswd ($record)) {
	return $error_message.$err;
      }
      if ($err = &VHConfiguration::SetLeastUsedRange ($record)) {
	return $error_message.$err;
      }
      return &ActualAddVirtUser($user,$fullname,$shell,$gid,$vroot);
    }
  }
  return $error_message;
}

# for name-based domains; add user to server
sub AddNBVirtUser() {
  my ($user,$fullname, $ftpenabled, $telnetenabled,$record)=@_;
  my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
  #get uid and gid of user in chrooted environment
  unless (open(PASSWD,"$vroot/etc/passwd")) {
     $cerr += $E_UNABLEREADPWDFILE and $cerr += { vroot => $vroot } ;
     $cerr += $E_USERADDERRORS ;
      return "Cannot read password file $vroot/etc/passwd.\nErrors during useradd.\n";
  }
  my (@lines)=<PASSWD>;
  close (PASSWD);
  my(@goodlines) = grep {$_ =~ /^$user:/} @lines;
  unless ($#goodlines == 0) {
    $cerr += $E_NODOMAINUSER and $cerr += { user => $user, domain => $record->{Domain} } ;
    $cerr += $E_USERADDERRORS ;
      return "User $user of domain $record->{Domain} does not exist.\nErrors during useradd.\n";
  }
  my (@pwent) = split(/:/,$goodlines[0]);
  my ($uid) = ($pwent[2]);
  my ($gid) = ($pwent[3]);
  my ($shell) = DetermineRootLevelShell($ftpenabled, $telnetenabled);
  #We add the user as user.siteownername

  #TODO:???:change the username to username.domainname

  my $dotuser = VHCommon::GetDotUser($user,$record);
  my $vhome = $vroot . "/home/$user";

  if($error_message = &ActualAddNBVirtUser($dotuser,$fullname,$shell,$uid,$gid, $vhome)) 

   {return $error_message;}
  SyncNBUser($dotuser, "$user", $record);

  #TODO:???:it is actually bad to do it here since it becomes hidden
  if ($record->{IMAP}) {
      AddNBMailBox("$user", $uid, $gid, $record);
  }
  return $error_message;
}

sub AddNBMailBox(){
  my ($user, $uid, $gid, $record) = @_;

  my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
  my ($chrootinbox)="$vroot/var/spool/mail/$user";

  my $dotuser = VHCommon::GetDotUser($user,$record);
  my ($inbox)="/var/spool/mail/$dotuser";

  if (system("/bin/ln -s $chrootinbox $inbox") && 0xff00) {
    $cerr += $E_UNABLECREATEMBOX and $cerr += { mbox => $inbox } ;
    $cerr += $E_ADDUSERMBOXERRORS and $cerr += { user => $user } ;
    return "Cannot create mail box $inbox.\nErrors during adding mail box for user $user.\n" ;
  }
  
  #somehow I have to make system call to chown symlink
  if (system("chown $uid.$gid $inbox") && 0xff00) {
    $cerr += $E_UNABLECHOWNMBOX and $cerr += { mbox => $chrootinbox } ;
    $cerr += $E_ADDUSERMBOXERRORS and $cerr += { user => $user } ;
    return "Cannot change ownership of mail box $chrootinbox .\nErrors during adding mail box for user $user.\n";
  }

}

sub SyncNBUser() {
    # syncs chrooted pwd to top level pwd.
    # assumes user already exists in chrooted env.
    my ($user,$chrootuser,$record)=@_;
    my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
    unless (open(SHADOW,"$vroot/etc/shadow")) {
        $cerr += $E_UNABLEOPENVROOTSHADOW and $cerr += { vroot => $vroot } ;
        return "Cannot open shadow file $vroot/etc/shadow.\n";
    }
    my (@shadow_lines)=<SHADOW>;
    close (SHADOW);
 
    my(@goodshadow_lines) = grep {$_ =~ /^$chrootuser:/} @shadow_lines;
    unless ($#goodshadow_lines == 0) {
        $cerr += $E_NODOMAINUSER and $cerr += { user => $chrootuser, domain => $record->{Domain} } ;
         return "User $chrootuser of domain $record->{Domain} does not exist.\n";
    }
 
    my (@shadow_pwent) = split(/:/,$goodshadow_lines[0]);
    my ($shadow_pwd) = ($shadow_pwent[1]);
    unless (open(CHPASSWD,"|/usr/sbin/chpasswd -e")) {
        $cerr += $E_UNABLECHANGESHADOWPWD and $cerr += { user => $user } ;
         return "Could not change /etc/shadow password for user $user.\n";
    }
    print CHPASSWD "$user:$shadow_pwd\n";
    close CHPASSWD;
}

sub ActualAddVirtUser() {
  my($user,$fullname,$shell,$gid,$vroot)=@_;
  # add user

  my($error_message)="";

  # obtain a uid
  # 

  $uid = get_free_uid();

  unless (open (ADDUSER, "/usr/sbin/chroot $vroot /usr/sbin/useradd -m -s $shell -u $uid -g $gid -c \'$fullname\' $user 2>&1 |"))
  #Solaris: unless (open (ADDUSER, "/usr/sbin/chroot $vroot /usr/sbin/useradd -s $shell -u uid -g $gid -c \'$fullname\' $user 2>&1 |"))
    {
    $cerr += $E_UNABLECHROOTUSERADD and $cerr += $E_USERADDERRORS;
    return "Cannot create the chrooted useradd process.\nErrors during useradd.\n"; }
  while (<ADDUSER>) {
    $error_message.=$_;
  }
  unless (close (ADDUSER))
    {
    $cerr += $E_USERADDERRORS ;
    $error_message.="Errors during useradd.\n" ;}
  if ($error_message) {
      return $error_message;
  }
  unless (chmod(0711,"$vroot/home/$user")) {
    $cerr += $E_UNABLECHMODHOMEDIR;
      return "Cannot change permissions on the user's home directory.\n";
  }
  my ($inbox)="$vroot/var/spool/mail/$user";
  unless (open (INBOX,">$inbox")) {
    $cerr += $E_UNABLECREATEMBOX and $cerr += { mbox => $inbox } ;
    $cerr += $E_USERADDERRORS ;
      return "Cannot create mail box $inbox .\nErrors during useradd.\n";
  }
  close (INBOX);
  unless (open(PASSWD,"$vroot/etc/passwd")) {
    $cerr += $E_UNABLEREADPWDFILE and $cerr += { vroot => $vroot } ;
    $cerr += $E_USERADDERRORS ;
      return "Cannot read password file $vroot/etc/passwd .\nErrors during useradd.\n";
  }
  my (@lines)=<PASSWD>;
  close (PASSWD);
  @lines = grep {$_ =~ /^$user:/} @lines;
  unless ($#lines == 0) {
    $cerr += $E_USERNOTADDED and $cerr += $E_USERADDERRORS ;
      return "User not properly added to password file.\nErrors during useradd.\n";
  }

  my (@pwent)=split(/:/,$lines[0]);
  unless (chown($pwent[2],$pwent[3], $inbox)) {
    $cerr += $E_UNABLECHOWNMBOX and $cerr += { mbox => $inbox } ;
    $cerr += $E_USERADDERRORS ;
      return "Cannot change ownership of mail box $inbox .\nErrors during useradd.\n";
  }
  unless (chmod(0600,$inbox)) {
    $cerr += $E_UNABLECHMODMBOX and $cerr += { mbox => $inbox } ;
    $cerr += $E_USERADDERRORS ;
      return "Cannot modify permissions of mail box $inbox .\nErrors during useradd.\n";
  }
  return "";
}

# for name-based domains; add user as username.domain with same uid
sub ActualAddNBVirtUser() {
  my($user,$fullname,$shell,$uid,$gid, $vhome)=@_;
  my($error_message)="";

  unless (open (ADDUSER, "/usr/sbin/real.useradd -m -s $shell -g $gid -u $uid -c \'$fullname\' \'$user\' 2>&1 |"))
    { $cerr += $E_UNABLERUNUSERADD and $cerr += $E_USERADDERRORS ;
    return "Cannot create the useradd process on server.\nErrors during useradd.\n"; }
  while (<ADDUSER>) {
    $error_message.=$_;
  }
  unless (close (ADDUSER))
    {
    $cerr += $E_USERADDSRVERR ;
    $error_message.="Errors during useradd on server.\n" ;}

  my(@pwent) = getpwnam($user);
  unless (@pwent) {
    $cerr += $E_INVALIDUSER and $cerr += { user => $user } ;
    return "Invalid user name $user .\n" ;}
  my($homedir) = $pwent[7];
  if (system ( "ln -s \'$vhome\' \'$homedir/siteroot\' " ))
  { 
    $cerr += $E_FTPPOINTERERROR and $cerr += { user => $user } ;
    $error_message.="Errors in creating ftp pointer for user $user.\n"; }

  if ($error_message) {
      return $error_message;
  }

  return "";
}

# delete a user from a chroot environment
sub DelVirtUser() {
  my ($user,$record)=@_;
  my ($error_message)="";
  my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
  #Here we do an otherwise useless check to avoid letting the domain
  #owners to delete important "virtual users" brought over from the
  #server (root, ftp, majordomo, domain owner, etc)
  unless(open (PASSWD, "$vroot/etc/passwd")) {
    $cerr += $E_UNABLEREADVIRTPWDFILE and $cerr += { vroot => $vroot } ;
    return "Cannot read virtual password file $vroot/etc/passwd .\n";
  }
  my($passwdline);
  while (<PASSWD>) {
      if ($_ =~ /^$user:/ ) {
	  $passwdline = $_;
	  last;
      }
  }
  close (PASSWD);
  if (not $passwdline) {
      #We should never get to this line
      $cerr += $E_USERNOTINSITE and $cerr += { user => $user, domain => $record->{Domain} } ;
      return "User $user not found in site $record->{Domain} .\n";
  }
  my(@pwent) = split(/:/,$passwdline);
  my($uid) = $pwent[2];
  if($error_message=&VHConfiguration::ReadUIDConfFile()) {
      return $error_message
  }
  if($uid<$VHConfiguration::MinVirtUid or $uid>$VHConfiguration::MaxVirtUid) {
    $cerr += $E_SPUSERDEL and $cerr += { user => $user } ;
      return "You are not allowed to delete special user $user .\n";
  }
  #End of stupid check
  unless (open (DELUSER, "/usr/sbin/chroot $vroot /usr/sbin/userdel -r $user 2>&1 |"))
    {   $cerr += $E_UNABLECHROOTUSERDEL ;
        return "Cannot create the chrooted deluser process.\n"; }
  while (<DELUSER>) {
    $error_message.=$_;
  }
  close (DELUSER) or $cerr += $E_USERDELERRORS and $error_message.="Errors during userdel.\n";

  free_uid($uid);

  my ($inbox)="$vroot/var/spool/mail/$user";
  if (stat($inbox)) {
    unlink ($inbox) or $cerr += $E_UNABLEDELETEINBOX and $cerr += { inbox => $inbox } and $error_message.= "Cannot delete inbox $inbox .\n";
  }
  print STDERR $error_message;
  #Now we delete the aliases that point to this user
  if (open (ALIAS,"$vroot/etc/aliases")) {
      my (@lines);
      @lines = <ALIAS>;
      close (ALIAS);
      my ($linenum);
      $linenum = $#lines;
      @lines = grep {$_ !~ /\s$user$/} @lines;
      if ($linenum != $#lines) {
          if (open (ALIAS,">$vroot/etc/aliases")) {
              print ALIAS @lines;
              close (ALIAS);
          } else {
            $cerr += $E_UNABLEWRITEALIASFILE and $cerr += { vroot => $vroot } ;
              print STDERR "Cannot write to alias file $vroot/etc/aliases .\n";
          }
      }
  } else {
    $cerr += $E_UNABLEREADALIASFILE and $cerr += { vroot => $vroot } ;
      print STDERR "Cannot read alias file $vroot/etc/aliases .\n";
  }

  return "";
}


# for name-based domains; delete the mail box of the user from the server
sub DelNBMailBox($$) {
  my $user = $_[0];
  my $record = $_[1];

  my ($error_message)="";
  #We have user.siteownername

  $dotuser = VHCommon::GetDotUser($user,$record);

  #Remove the symlink to the virtual user's mail box 
  my ($inbox)="/var/spool/mail/$dotuser";
  # this file is a symlink, so we need lstat
  if (lstat($inbox)) {
    unlink ($inbox) or $cerr += $E_UNABLEDELETEINBOX and 
        $cerr += { inbox => $inbox } and 
        $error_message.= "Cannot delete inbox $inbox .\n";
  }

  if ($error_message) {print STDERR $error_message;}
  return "";
}

# for name-based domains; delete a user from the server
# this is called only when the virtual user is also deleted from the chrooted environment
sub DelNBVirtUser($$) {
  my $user =$_[0];
  my $record =$_[1];
  my ($error_message)="";

  #We switch to user.siteownername
  
  my $dotuser = VHCommon::GetDotUser($user,$record);

  unless (open (DELUSER, "/usr/sbin/userdel -r $dotuser 2>&1 |"))
    { $cerr += $E_UNABLEUSERDEL ;
    return "Cannot create the deluser process.\n";}
  while (<DELUSER>) {
    $error_message.=$_;
  }
  close (DELUSER) or $cerr += $E_USERDELERRORS and 
    $error_message.="Errors during userdel.\n";
  if ($error_message) {print STDERR $error_message;}

  my(@pwent) = getpwnam($dotuser);
  unless (@pwent) { $cerr += $E_INVALIDUSER and $cerr += { user => $dotuser } and return "Invalid user name $dotuser .\n" ;}
  my($homedir) = $pwent[7]; 

  #remove the mailbox of the user from the server
  DelNBMailBox($user,$record);

  return "";
}

sub GetDotUser($$) {
    my($puser,$record)=@_;

    #if domain owner, return the original name
    if ("$record->{User}" eq "$puser") {
	return "$puser";
    }
    #if virtual user, return $user.$domainname
    my $str = $record->{Domain};
    my $token;
    my $ext = "";
    my (@fields) = split(/\./,$str);
    foreach $token (@fields) {
	$ext = "$ext" . "$token";
    }
    return "$puser\.$ext";
}

# change the shell of a virtual user
sub ChShellVirtUser() {
  my ($user, $ftp, $telnet, $record)=@_;
  my $vroot = "$VHConst::VIRTUALFS/$record->{User}";
  my $err ="";

  my $vshell = VHCommon::DetermineShell($ftp,$telnet);

  if( system("/usr/sbin/chroot $vroot /usr/sbin/usermod -s $vshell $user") ) {
        $cerr += $E_USERCHSHPROB and $cerr += { user => $user } ;
      $err = $err . "Problem in changing the shell of user $user\n";
  }
  
  # for name-based domains; change the shell of a user on server
   if (Need_Root_SiteUser($record)) {
      my $dotuser = VHCommon::GetDotUser($user,$record);
      my $shell = DetermineRootLevelShell($ftp,$telnet);
      if( system("/usr/sbin/usermod -s $shell $dotuser") ) {
        $cerr += $E_USERCHSHPROB and $cerr += { user => $dotuser } ;
	  $err = $err . "Problem in changing the shell of user $dotuser \n";
      }      
      return $err;
  }
}

# change the fullname of a virtual user
sub ChfullnameVirtUser() {
  my ($user,$fullname,$record)=@_;

  my $vroot = "$VHConst::VIRTUALFS/$record->{User}";

  if( system("/usr/sbin/chroot $vroot /usr/sbin/usermod -c \'$fullname\' \'$user\'") ) {
        $cerr += $E_USERFULLNAMEPROB and $cerr += { user => $user } ;
      return "Problem in changing the full name of user $user\n";
  }
  
  # for name-based domains; change the fullname of a user on server
   if (Need_Root_SiteUser($record)) {
      my $dotuser = VHCommon::GetDotUser($user,$record);
      if( system("/usr/sbin/usermod -c \'$fullname\' \'$dotuser\'") ) {
        $cerr += $E_USERFULLNAMEPROB and $cerr += { user => $user } ;
	  return "Problem in changing the full name of user $dotuser \n";
      }      
      return "";
  }
}

# change the password of a virtual user
sub ChpasswdVirtUser() {
  my ($user,$passwd,$record)=@_;
  my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
  my ($error_message)="";

  unless(open (CHPASSWD, "|/usr/sbin/chroot $vroot /usr/sbin/chpasswd"))
    { $cerr += $E_UNABLECHROOTCHPWD ;
    return "Cannot create the chrooted chpasswd process.\n" ;}
  print CHPASSWD "$user:$passwd\n";
  unless (close (CHPASSWD))
    { $cerr += $E_CHPASSWDCHROOTERR ;
    $error_message.="Errors during chpasswd in chrooted.\n";}
  # for name-based domains; change the password of a user on server
   if (Need_Root_SiteUser($record)) {
      my $dotuser = VHCommon::GetDotUser($user,$record);
      unless(open (CHPASSWD, "|/usr/sbin/chpasswd"))
        { $cerr += $E_UNABLECHPWD ;
        return "Cannot creat the chpasswd process.\n";}
      print CHPASSWD "$dotuser:$passwd\n";
      unless(close (CHPASSWD))
        { $cerr += $E_CHPASSWDERRSRV ;
        $error_message.="Errors during chpasswd on server.\n";}
  }
  return $error_message;
}

# change the password of a user from a chroot environment
sub ChpasswdDomain() {
  my ($record,$passwd)=@_;
  my ($user) = $record->{User};

  #Change it on the server first
  unless(open (CHPASSWD, "|/usr/sbin/chpasswd"))
    { $cerr += $E_UNABLECHPWD ;
    return "Cannot create the chpasswd process.\n" ;}
  print CHPASSWD "$user:$passwd\n";
  unless (close (CHPASSWD))
    { $cerr += $E_CHPASSWDERR ;
    return "Errors during chpasswd.\n";}

  #and in the chrooted environment afterwards
  my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
  unless(open (CHPASSWD, "|/usr/sbin/chroot $vroot /usr/sbin/chpasswd")){
    $cerr += $E_UNABLECHROOTCHPWD ;
      print STDERR "Cannot create the chrooted chpasswd process.\n";
      return "";
  }
  print CHPASSWD "$user:$passwd\n";
  unless (close (CHPASSWD))
    { $cerr += $E_CHPASSWDERRVUSR ;
    print STDERR "Errors during chpasswd for the virtual user.\n"; }

  #and if frontpage is installed, change it there too
  if ($record->{FPExt}) {
      if ($error_message = &VHAppConf::resetFrontPagePassword($record->{User})) {
        $cerr += $E_UNABLECHANGEFRONTPAGEPWD ;
          return "Failed to change the associated frontpage password\n".$error_message;
      }
  }

  return "";
}

# Gives the unix users shell based on the flags. At the box level, we never give bash
sub DetermineRootLevelShell() {
  my ($ftp,$telnet)=@_;
  my $shell = DetermineShell($ftp, $telnet);
  if ( $shell eq "/bin/bash" ) {
      return "/bin/virtshell";
  }
  return $shell;
}

# Gives he shell a virtual user gets based on its flags
sub DetermineShell() {
  my ($ftp,$telnet)=@_;
  
  #if telnet is enabled, ftp must be enabled also
  if ($telnet) { return "/bin/bash"; }
  if ($ftp) { return "/bin/ftponly"; }
  return "/bin/nologinshell"; 
}

sub GetVRoot(){ 
  my $record = $_[0];
  my $str = "$VHConst::VIRTUALFS/$record->{User}" ;
  return $str ;
}

# Formats a line from the virtual passwd file
sub FormatVirtUser() {
  my ($admin, $telnetenabled,$ftpenabled,$domaintype,$line,$used,$quota)=@_;
  chop ($line);
  my (@fields) = split(/:/,$line);
  my $username = $fields[0];
  my ($shell) = $fields[6];
  my ($ftp) = ((grep {$shell eq $_}("/bin/bash","/bin/nologinshell","/bin/false","/dev/null"))?0:1);
  if ($domaintype eq "NameBased") {$ftp=0;}
  if (!$ftpenabled) {$ftp=0;}
  my ($telnet) = (($shell eq "/bin/bash")?"1":"0");
  if (!$telnetenabled) {$telnet=0;}
  if (not defined $quota) {$quota="";}
  if (not defined $used) {$used="";}
  my($uid) = $fields[2];
  my($error_message,$privileged);
  
  if ($username eq $admin) {
      $privileged=1;
  } else {
      $privileged=0;
  }    

  return "$fields[0]:$ftp:$telnet:$privileged:$quota:$used:$fields[4]\n";
}

#
# This little shrewd function cleans up a virtual domain so that
# telnet can be disabled. For this, it kills all the processes
# belonging to virtual users who telneted in. After this it unmounts
# /dev/pts so that noone else gets in.
#
sub PrepareDisableTelnet() {
    my ($record)=@_;
    my (@victims)=();

    my $dir = "$VHConst::VIRTUALFS/$record->{User}";

    # 'ps | grep $dir' doesnt work for Name Based domains since we dont mount 
    # the chrooted /dev/pts to the box
    #open(PROCESSES,'/bin/ps -Ao "%p %y"|') or return "Cannot do ps.\n";

    #TODO:???:this is still not working
    open(PROCESSES, "ls -l /proc/*/cwd 2>/dev/null | grep \'$dir\' | sed 's/.*\/proc\/\([[:alnum:]]\+\).*/\1/' |") or $cerr += $E_UNABLEREADPROCESSES and return "Cannot read the processes.\n";
    while(<PROCESSES>){
	push @victims, $_;
    }    
    kill 9,@victims;

    #for NB domains, we dont need to mount/umount, since we use 
    #the /dev/pts on the box
    if ($record->{DomainType} eq "NameBased") { 
	return "";
    }

    system ("/bin/umount $dir/dev/pts >/dev/null 2>/dev/null");
    if (($? >> 8) != 0) {
    $cerr += $E_UNMOUNTFSERR and $cerr += { dir => "$dir/dev/pts" } ;
	return "Error unmounting file system $dir/dev/pts .\n";
    }
    return"";
}

#
# This function checks if all the services marked as on for a virtual
# domain are actually supported.
#
sub CheckSupportedServices() {
    my ($record)=@_;

    my($analog_supported,$ssl_supported,$imap_supported,$vacation_supported,
       $majordomo_supported,$sendmail_supported,$telnet_supported,
       $bind_supported);
    open (SVCINST,"/etc/virtualhosting/supported_services") or
        $cerr += $E_UNABLEOPENINSTWITNESS and 
      return "Cannot open service installation witness file.\n";
    open (SVCCRT,"/usr/bin/appsvcquery -a|") or
        $cerr += $E_UNABLERUN and 
        $cerr += { what => "/usr/bin/appsvcquery" } and
      return "Cannot run /usr/bin/appsvcquery .\n";
    my (@svcinst)=<SVCINST>;
    my (@svccrt)=<SVCCRT>;
    close (SVCINST);
    close (SVCCRT);
    if ((grep /^analog\s/,@svcinst) and (grep /^analog\n$/,@svccrt)) {
	$analog_supported = 1;
    } else {
	$analog_supported = 0;
    }
    if ((grep /^openssl\s/,@svcinst) and (grep /^openssl\n$/,@svccrt)) {
	$ssl_supported = 1;
    } else {
	$ssl_supported = 0;
    }
#    if ((grep /^sslc\s/,@svcinst) and (grep /^sslc\n$/,@svccrt)) {
#	$ssl_supported = 1;
#    } else {
#	$ssl_supported = 0;
#    }
    if ((grep /^imap\s/,@svcinst) and (grep /^imap\n$/,@svccrt)) {
	$imap_supported = 1;
    } else {
	$imap_supported = 0;
    }
    if ((grep /^majordomo\s/,@svcinst) and (grep /^majordomo\n$/,@svccrt)) {
	$majordomo_supported = 1;
    } else {
	$majordomo_supported = 0;
    }
    if ((grep /^vacation\s/,@svcinst) and (grep /^vacation\n$/,@svccrt)) {
	$vacation_supported = 1;
    } else {
	$vacation_supported = 0;
    }
    if ((grep /^sendmail\s/,@svcinst) and (grep /^sendmail\n$/,@svccrt)) {
	$sendmail_supported = 1;
    } else {
	$sendmail_supported = 0;
    }
    if ((grep /^bind\s/,@svcinst) and (grep /^bind\n$/,@svccrt)) {
	$bind_supported = 1;
    } else {
	$bind_supported = 0;
    }
    $telnet_supported = 1;
    my ($error_message)="";
    if ($record->{Analog} and not $analog_supported) {
	$error_message .= "Option Analog not supported\n";
    }
    if ($record->{SSL} and not $ssl_supported) {
    $cerr += $E_OPTNOTSUPPORTED and $cerr += { opt => "SSL" };
	$error_message .= $record->{SSL} . "$ssl_supported". "Option SSL not supported\n";
    }
    if ($record->{Majordomo} and not $majordomo_supported) {
    $cerr += $E_OPTNOTSUPPORTED and $cerr += { opt => "Majordomo" };
	$error_message .= "Option Majordomo not supported\n";
    }
    if ($record->{Vacation} and not $vacation_supported) {
    $cerr += $E_OPTNOTSUPPORTED and $cerr += { opt => "Vacation" };
	$error_message .= "Option Vacation not supported\n";
    }
    if ($record->{Sendmail} and not $sendmail_supported) {
    $cerr += $E_OPTNOTSUPPORTED and $cerr += { opt => "Sendmail" };
	$error_message .= "Option Sendmail not supported\n";
    }
    if ($record->{Telnet} and not $telnet_supported) {
    $cerr += $E_OPTNOTSUPPORTED and $cerr += { opt => "Analog" };
	$error_message .= "Option Analog not supported\n";
    }
    if ($bind_supported) {
	$record->{Nslookup} = 1;
    } else {
	$record->{Nslookup} = 0;
    }
    return $error_message;
}

#
# This function checks if a service is installed and supported on the server
#
sub IsServiceInstalledSupported() {
    my ($service)=@_;
    # when we standardize the naming on backend/frontend,we'll be able
    # to get rid of this junk...
    my ($svc_name); # name of service as it is running on server
    if ($service eq "Web") { $svc_name="apache"; }
    elsif ($service eq "Analog") { $svc_name="analog"; }
    elsif ($service eq "Nslookup") { $svc_name="bind"; }
    elsif ($service eq "FPext") { $svc_name="frontpage"; }
    elsif ($service eq "Sendmail") { $svc_name="sendmail"; }
    elsif ($service eq "IMAP") { $svc_name="imap"; }
    elsif ($service eq "Majordomo") { $svc_name="majordomo"; }
    elsif ($service eq "FTP") { $svc_name="proftpd"; }
    elsif ($service eq "SSL") { $svc_name="openssl"; }
    elsif ($service eq "Vacation") { $svc_name="vacation"; }
    elsif ($service eq "Telnet") { $svc_name="telnet"; }
    elsif ($service eq "MivaMerchant") { $svc_name="mivamerchant"; }
    else { $svc_name=$service; }
    open (SVCINST,"/etc/virtualhosting/supported_services") or
        $cerr += $E_UNABLEOPENINSTWITNESS and
      return "Cannot open service installation witness file.\n";
    open (SVCCRT,"/usr/bin/appsvcquery -a|") or
        $cerr += $E_UNABLERUN and 
        $cerr += { what => "/usr/bin/appsvcquery" } and
      return "Cannot run /usr/bin/appsvcquery .\n";
    my (@svcinst)=<SVCINST>;
    my (@svccrt)=<SVCCRT>;
    close (SVCINST);
    close (SVCCRT);
    if ((grep /^$svc_name\s/,@svcinst) and (grep /^$svc_name\n$/,@svccrt)) {
	return "";
    } else {
        $cerr += $E_OPTNOTSUPPORTED and $cerr += { opt => $service } ;
	return "Option $service not supported.\n";
    }
}

sub Need_Root_SiteUser($){
    
    # if we support any of these options for NB site users, 
    # we need the user to have account at the box also

    my ($record) = @_;    

    if ($record->{DomainType} eq "NameBased") {return 1;};
    return 0;
}

sub get_free_uid() {
    my $uid;
    open(FUIDS, "+< /etc/virtualhosting/free-uids");
    flock(FUIDS, Fcntl::LOCK_EX);
    seek(FUIDS, -6, Fcntl::SEEK_END);
    read(FUIDS, $uid, 5);
    @stat = stat(FUIDS);
    truncate(FUIDS, $stat[7] - 6);
    flock(FUIDS, Fcntl::LOCK_UN);
    close(FUIDS);
    return $uid;
}

sub free_uid($) {
    $uid = $_[0];
    open(FUIDS, ">> /etc/virtualhosting/free-uids");
    flock(FUIDS, Fcntl::LOCK_EX);
    print FUIDS "$uid\n";
    flock(FUIDS, Fcntl::LOCK_UN);
    close(FUIDS);
}

return 1;

