#!/usr/bin/ensim-python
import sys
import getopt
import commands
import ensimapplpath
from vh3 import virthost
import egv
from SystemState import SystemState, EStateTransition

def usage(err_code):
    sys.stderr.write("""
Usage:
    %s [-h|--help] [-f|--filename=<license file>] [-t|--type=<Manage Type>]

    where <license file> is the full path to the license file
    and   <Manage Type>  is either 'SXC' or 'SA'

""" % sys.argv[0])
    sys.exit(err_code)

def crib(message, manage_type):
    if manage_type == 'SXC':
        sys.stderr.write('@@VK:WARNING:%s\n' % message)
    else:
        sys.stderr.write('%s\n' % message)
    sys.exit(1)

def main():
    LICENSE_ENC = "/etc/appliance/.license/license.xml"
    filename = None
    Mtype = None   # Default to stderr

    # Parse cmdline
    short_opts = 'hf:t:'
    long_opts  = ['help', 'filename=', 'type=']
    try:
        opts, args = getopt.getopt(sys.argv[1:], short_opts, long_opts)
    except getopt.GetoptError:
        # print usage and exit
        usage(2)
    for o, a in opts:
        if o in ('-h', '--help'):
            usage(1)
        elif o in ('-f', '--filename'):
            filename = a
        elif o in ('-t', '--type'):
            if a not in ('SXC', 'SA'):
                usage(1)
            Mtype = a
    if None in (filename, Mtype):
        usage(1)

    # who am i ??
    #initial = SystemState()
    try:
        # Validate the license file 
        ptlicense = egv.HLicense(filename)
        #[rc, [msg, expl]] = egv.validate(0,filename)
        [rc, [msg, expl]] = ptlicense.validate()
        if rc: 
            raise egv.ELicense([rc, [msg, expl]])
            
        # what will I become ?
        #final = SystemState(filename)
        
        # Since I'm here, ie: no exceptions were raised, I can update myself to
        # what I want to become.
        #initial.updateState(final)
        #initial.commit()
    except egv.ELicense, msg:
        # initial.save_state() 
        crib(msg, Mtype)
    except EStateTransition, msg:
        # initial.save_state()
        crib(msg, Mtype)
    else:
        # Save the license file 
        ret, err = commands.getstatusoutput('cp -f %s %s' % (filename, LICENSE_ENC))
        if ret:
            crib("""
                Applied the license successfully by could not copy the license file.
                Please copy the file manually to %s.
                Error: %s""" % (LICENSE_ENC, err), Mtype)
        #return initial.save_state()

if __name__ == '__main__':
    main()
